﻿
assert(TooltipExchange, "TooltipExchange not found!")

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("TooltipExchangeLoot")
local dewdrop = AceLibrary("Dewdrop-2.0")
local periodic = AceLibrary("PeriodicTable-3.0")

-------------------------------------------------------------------------------
-- Localization
-------------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Raid Loot"] = true,
	["Raid instance bosses drop lists."] = true,
	["All instance loot"] = true,
	["Displays all item that drop in given instance."] = true,
} end)

-------------------------------------------------------------------------------
-- Declaration
-------------------------------------------------------------------------------

local instanceLoot = {
	{ header = L["Raid Loot"] },
	{ text = "Azeroth", zone = "World Bosses", bosses = { "Azuregos", "Lord Kazzak", "Emeriss", "Lethon", "Taerar", "Ysondre" } },
	{ zone = "Zul'Gurub" },
	{ zone = "Ruins of Ahn'Qiraj" },
	{ zone = "Onyxia's Lair" },
	{ zone = "Molten Core" },
	{ zone = "Blackwing Lair" },
	{ zone = "Ahn'Qiraj" },
	{ zone = "Naxxramas" },
	{ header = " " },
	{ text = "Outland", zone = "World Bosses", bosses = { "Doom Lord Kazzak", "Doomwalker" } },
	{ zone = "Karazhan" },
	{ zone = "Gruul's Lair" },
	{ zone = "Magtheridon's Lair" },
	{ zone = "Serpentshrine Cavern" },
	{ text = "Tempest Keep", zone = "The Eye" },
}

TooltipExchangeLoot = TooltipExchange:NewModule(L["Raid Loot"])
TooltipExchangeLoot.notToggable = true

TooltipExchangeLoot.commands = {
	raidLoot = {
		type = "group",
		name = L["Raid Loot"],
		desc = L["Raid instance bosses drop lists."],
		order = 15,
		args = {},
	},
}

TooltipExchangeLoot.revision = tonumber(string.sub("$Revision: 30528 $", 12, -3))

-------------------------------------------------------------------------------
-- Main
-------------------------------------------------------------------------------

function TooltipExchangeLoot:OnInitialize()
	self:CreateStaticMenu()
end

function TooltipExchangeLoot:OnEnable()

end

function TooltipExchangeLoot:OnDisable()

end

-------------------------------------------------------------------------------
-- Dynamic Menu
-------------------------------------------------------------------------------

function TooltipExchangeLoot:CreateStaticMenu()
	local a, o = self.commands.raidLoot.args, 100

	for _, v in ipairs(instanceLoot) do
		if v.header then
			a["header" .. o] = {
				type = "header",
				order = o,
				name = v.header,
			}
		elseif v.zone then
			local zz = string.gsub(v.text or v.zone, "[^%a]", "")
			local z, c = v.zone, 0

			a[zz] = {
				type = "group",
				name = v.text or v.zone,
				desc = v.text or v.zone,
				order = o,
				args = {
					header = {
						type = "header",
						order = 1,
						name = v.text or v.zone,
					},
				},
			}

			if v.bosses then
				for _, b in ipairs(v.bosses) do
					a[zz].args[string.gsub(b, "[^%a]", "")] = {
						type = "execute",
						name = b,
						desc = b,
						func = function()
							self:TriggerEvent("TooltipExchange_PeriodicResults", "InstanceLoot." .. z .. "." .. b)
							dewdrop:Close()
						end,
					}
					c = c + 1
				end
			else
				local t = periodic:GetSetTable("InstanceLoot." .. z)

				if t then
					for _, set in ipairs(t) do
						if set.set then
							local b = string.sub(set.set, string.len("InstanceLoot." .. z) + 2)

							a[zz].args[string.gsub(b, "[^%a]", "")] = {
								type = "execute",
								name = b,
								desc = b,
								func = function()
									self:TriggerEvent("TooltipExchange_PeriodicResults", "InstanceLoot." .. z .. "." .. b)
									dewdrop:Close()
								end,
							}
							c = c + 1
						end
					end

					if c > 1 then
						a[zz].args.allInstanceLoot = {
							type = "execute",
							name = L["All instance loot"],
							desc = L["Displays all item that drop in given instance."],
							func = function()
								self:TriggerEvent("TooltipExchange_PeriodicResults", "InstanceLoot." .. z)
								dewdrop:Close()
							end,
							order = -1,
						}
					end
				end
			end

			if c == 0 then
				a[zz] = nil
			end
		end

		o = o + 1
	end
end
